@@ -1,13 +1,17 @@
-use actix_web::http::StatusCode;
-use chrono::{DateTime, Utc};
-use chrono_humanize::Humanize;
-use clap::{crate_name, crate_version};
-use fast_qr::convert::svg::SvgBuilder;
-use fast_qr::qr::QRCodeError;
-use fast_qr::QRBuilder;
-use maud::{html, Markup, PreEscaped, DOCTYPE};
 use std::time::SystemTime;
-use strum::IntoEnumIterator;
+
+use actix_web::http::StatusCode;
+use chrono::{DateTime, Local};
+use chrono_humanize::Humanize;
+use clap::{crate_name, crate_version, ValueEnum};
+use fast_qr::{
+    convert::{svg::SvgBuilder, Builder},
+    qr::QRCodeError,
+    QRBuilder,
+};
+use http::Uri;
+use maud::{html, Markup, PreEscaped, DOCTYPE};
+use strum::{Display, IntoEnumIterator};
 
 use crate::auth::CurrentUser;
 use crate::consts;
@@ -19,7 +23,7 @@ use crate::{archive::ArchiveMethod, MiniserveConfig};
 pub fn page(
     entries: Vec<Entry>,
     readme: Option<(String, String)>,
-    abs_url: impl AsRef<str>,
+    abs_uri: &Uri,
     is_root: bool,
     query_params: QueryParameters,
     breadcrumbs: &[Breadcrumb],
@@ -44,7 +48,7 @@ pub fn page(
         || conf
             .allowed_upload_dir
             .iter()
-            .any(|x| encoded_dir.starts_with(&format!("/{}", x)));
+            .any(|x| encoded_dir.starts_with(&format!("/{x}")));
 
     html! {
         (DOCTYPE)
@@ -97,7 +101,7 @@ pub fn page(
                     }
                 }
                 nav {
-                    (qr_spoiler(conf.show_qrcode, abs_url))
+                    (qr_spoiler(conf.show_qrcode, abs_uri))
                     (color_scheme_selector(conf.hide_theme_selector))
                 }
                 div.container {
@@ -190,7 +194,7 @@ pub fn page(
                     }
                     div.footer {
                         @if conf.show_wget_footer {
-                            (wget_footer(&title_path, current_user))
+                            (wget_footer(abs_uri, conf.title.as_deref(), current_user.map(|x| &*x.name)))
                         }
                         @if !conf.hide_version_footer {
                             (version_footer())
@@ -237,8 +241,8 @@ pub fn raw(entries: Vec<Entry>, is_root: bool) -> Markup {
 }
 
 /// Renders the QR code SVG
-fn qr_code_svg(url: impl AsRef<str>, margin: usize) -> Result<String, QRCodeError> {
-    let qr = QRBuilder::new(url.as_ref().into())
+fn qr_code_svg(url: &Uri, margin: usize) -> Result<String, QRCodeError> {
+    let qr = QRBuilder::new(url.to_string())
         .ecl(consts::QR_EC_LEVEL)
         .build()?;
     let svg = SvgBuilder::default().margin(margin).to_str(&qr);
@@ -264,26 +268,39 @@ fn version_footer() -> Markup {
     }
 }
 
-fn wget_footer(title_path: &str, current_user: Option<&CurrentUser>) -> Markup {
-    let count = {
-        let count_slashes = title_path.matches('/').count();
-        if count_slashes > 0 {
-            count_slashes - 1
-        } else {
-            0
-        }
+fn wget_footer(abs_path: &Uri, root_dir_name: Option<&str>, current_user: Option<&str>) -> Markup {
+    fn escape_apostrophes(x: &str) -> String {
+        x.replace('\'', "'\"'\"'")
+    }
+
+    // Directory depth, 0 is root directory
+    let cut_dirs = match abs_path.path().matches('/').count() - 1 {
+        // Put all the files in a folder of this name
+        0 => format!(
+            " -P '{}'",
+            escape_apostrophes(
+                root_dir_name.unwrap_or_else(|| abs_path.authority().unwrap().as_str())
+            )
+        ),
+        1 => String::new(),
+        // Avoids putting the files in excessive directories
+        x => format!(" --cut-dirs={}", x - 1),
     };
 
-    let user_params = if let Some(user) = current_user {
-        format!(" --ask-password --user {}", user.name)
-    } else {
-        "".to_string()
+    // Ask for password if authentication is required
+    let user_params = match current_user {
+        Some(user) => format!(" --ask-password --user '{}'", escape_apostrophes(user)),
+        None => String::new(),
     };
 
+    let command =
+        format!("wget -rcnHp -R 'index.html*'{cut_dirs}{user_params} '{abs_path}?raw=true'");
+    let click_to_copy = format!("navigator.clipboard.writeText(\"{command}\")");
+
     html! {
         div.downloadDirectory {
             p { "Download folder:" }
-            div.cmd { (format!("wget -r -c -nH -np --cut-dirs={} -R \"index.html*\"{} \"http://{}/?raw=true\"", count, user_params, title_path)) }
+            a.cmd title="Click to copy!" style="cursor: pointer;" onclick=(click_to_copy) { (command) }
         }
     }
 }
@@ -295,7 +312,7 @@ fn build_upload_action(
     sort_method: Option<SortingMethod>,
     sort_order: Option<SortingOrder>,
 ) -> String {
-    let mut upload_action = format!("{}?path={}", upload_route, encoded_dir);
+    let mut upload_action = format!("{upload_route}?path={encoded_dir}");
     if let Some(sorting_method) = sort_method {
         upload_action = format!("{}&sort={}", upload_action, &sorting_method);
     }
@@ -308,7 +325,7 @@ fn build_upload_action(
 
 /// Build the action of the mkdir form
 fn build_mkdir_action(mkdir_route: &str, encoded_dir: &str) -> String {
-    format!("{}?path={}", mkdir_route, encoded_dir)
+    format!("{mkdir_route}?path={encoded_dir}")
 }
 
 const THEME_PICKER_CHOICES: &[(&str, &str)] = &[
@@ -319,20 +336,30 @@ const THEME_PICKER_CHOICES: &[(&str, &str)] = &[
     ("Monokai (dark)", "monokai"),
 ];
 
-pub const THEME_SLUGS: &[&str] = &["squirrel", "archlinux", "zenburn", "monokai"];
+#[derive(Debug, Clone, ValueEnum, Display)]
+pub enum ThemeSlug {
+    #[strum(serialize = "squirrel")]
+    Squirrel,
+    #[strum(serialize = "archlinux")]
+    Archlinux,
+    #[strum(serialize = "zenburn")]
+    Zenburn,
+    #[strum(serialize = "monokai")]
+    Monokai,
+}
 
 /// Partial: qr code spoiler
-fn qr_spoiler(show_qrcode: bool, content: impl AsRef<str>) -> Markup {
+fn qr_spoiler(show_qrcode: bool, content: &Uri) -> Markup {
     html! {
         @if show_qrcode {
             div {
                 p {
                     "QR code"
                 }
-                div.qrcode #qrcode title=(PreEscaped(content.as_ref())) {
+                div.qrcode #qrcode title=(PreEscaped(content.to_string())) {
                     @match qr_code_svg(content, consts::SVG_QR_MARGIN) {
                         Ok(svg) => (PreEscaped(svg)),
-                        Err(err) => (format!("QR generation error: {:?}", err)),
+                        Err(err) => (format!("QR generation error: {err:?}")),
                     }
                 }
             }
@@ -378,7 +405,7 @@ fn archive_button(
     sort_order: Option<SortingOrder>,
 ) -> Markup {
     let link = if sort_method.is_none() && sort_order.is_none() {
-        format!("?download={}", archive_method)
+        format!("?download={archive_method}")
     } else {
         format!(
             "{}&download={}",
@@ -398,10 +425,10 @@ fn archive_button(
 
 /// Ensure that there's always a trailing slash behind the `link`.
 fn make_link_with_trailing_slash(link: &str) -> String {
-    if link.ends_with('/') {
+    if link.is_empty() || link.ends_with('/') {
         link.to_string()
     } else {
-        format!("{}/", link)
+        format!("{link}/")
     }
 }
 
@@ -439,8 +466,8 @@ fn build_link(
     sort_method: Option<SortingMethod>,
     sort_order: Option<SortingOrder>,
 ) -> Markup {
-    let mut link = format!("?sort={}&order=asc", name);
-    let mut help = format!("Sort by {} in ascending order", name);
+    let mut link = format!("?sort={name}&order=asc");
+    let mut help = format!("Sort by {name} in ascending order");
     let mut chevron = chevron_up();
     let mut class = "";
 
@@ -449,8 +476,8 @@ fn build_link(
             class = "active";
             if let Some(order) = sort_order {
                 if order.to_string() == "asc" {
-                    link = format!("?sort={}&order=desc", name);
-                    help = format!("Sort by {} in descending order", name);
+                    link = format!("?sort={name}&order=desc");
+                    help = format!("Sort by {name} in descending order");
                     chevron = chevron_down();
                 }
             }
@@ -504,7 +531,7 @@ fn entry_row(
                         @if !raw {
                             @if let Some(size) = entry.size {
                                 span.mobile-info.size {
-                                    (size)
+                                    (maud::display(size))
                                 }
                             }
                         }
@@ -513,15 +540,13 @@ fn entry_row(
             }
             td.size-cell {
                 @if let Some(size) = entry.size {
-                    (size)
+                    (maud::display(size))
                 }
             }
             td.date-cell {
-                @if let Some(modification_date) = convert_to_utc(entry.last_modification_date) {
+                @if let Some(modification_date) = convert_to_local(entry.last_modification_date) {
                     span {
-                        (modification_date.0) " "
-                        span.at { " at " }
-                        (modification_date.1) " "
+                        (modification_date) " "
                     }
                 }
                 @if let Some(modification_timer) = humanize_systemtime(entry.last_modification_date) {
@@ -611,15 +636,10 @@ fn page_header(title: &str, file_upload: bool, favicon_route: &str, css_route: &
 }
 
 /// Converts a SystemTime object to a strings tuple (date, time)
-/// Date is formatted as %e %b, e.g. Jul 12
-/// Time is formatted as %R, e.g. 22:34
-fn convert_to_utc(src_time: Option<SystemTime>) -> Option<(String, String)> {
-    src_time.map(DateTime::<Utc>::from).map(|date_time| {
-        (
-            date_time.format("%b %e").to_string(),
-            date_time.format("%R").to_string(),
-        )
-    })
+fn convert_to_local(src_time: Option<SystemTime>) -> Option<String> {
+    src_time
+        .map(DateTime::<Local>::from)
+        .map(|date_time| date_time.format("%Y-%m-%d %H:%M:%S %:z").to_string())
 }
 
 /// Converts a SystemTime to a string readable by a human,
@@ -677,3 +697,69 @@ pub fn render_error(
         }
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use pretty_assertions::assert_eq;
+
+    fn to_html(wget_part: &str) -> String {
+        format!(
+            r#"<div class="downloadDirectory"><p>Download folder:</p><a class="cmd" title="Click to copy!" style="cursor: pointer;" onclick="navigator.clipboard.writeText(&quot;wget -rcnHp -R 'index.html*' {wget_part}/?raw=true'&quot;)">wget -rcnHp -R 'index.html*' {wget_part}/?raw=true'</a></div>"#
+        )
+    }
+
+    fn uri(x: &str) -> Uri {
+        Uri::try_from(x).unwrap()
+    }
+
+    #[test]
+    fn test_wget_footer_trivial() {
+        let to_be_tested: String = wget_footer(&uri("https://github.com/"), None, None).into();
+        let expected = to_html("-P 'github.com' 'https://github.com");
+        assert_eq!(to_be_tested, expected);
+    }
+
+    #[test]
+    fn test_wget_footer_with_root_dir() {
+        let to_be_tested: String = wget_footer(
+            &uri("https://github.com/svenstaro/miniserve/"),
+            Some("Miniserve"),
+            None,
+        )
+        .into();
+        let expected = to_html("--cut-dirs=1 'https://github.com/svenstaro/miniserve");
+        assert_eq!(to_be_tested, expected);
+    }
+
+    #[test]
+    fn test_wget_footer_with_root_dir_and_user() {
+        let to_be_tested: String = wget_footer(
+            &uri("http://1und1.de/"),
+            Some("1&1 - Willkommen!!!"),
+            Some("Marcell D'Avis"),
+        )
+        .into();
+        let expected = to_html("-P '1&amp;1 - Willkommen!!!' --ask-password --user 'Marcell D'&quot;'&quot;'Avis' 'http://1und1.de");
+        assert_eq!(to_be_tested, expected);
+    }
+
+    #[test]
+    fn test_wget_footer_escaping() {
+        let to_be_tested: String = wget_footer(
+            &uri("http://127.0.0.1:1234/geheime_dokumente.php/"),
+            Some("Streng Geheim!!!"),
+            Some("uøý`¶'7ÅÛé"),
+        )
+        .into();
+        let expected = to_html("--ask-password --user 'uøý`¶'&quot;'&quot;'7ÅÛé' 'http://127.0.0.1:1234/geheime_dokumente.php");
+        assert_eq!(to_be_tested, expected);
+    }
+
+    #[test]
+    fn test_wget_footer_ip() {
+        let to_be_tested: String = wget_footer(&uri("http://127.0.0.1:420/"), None, None).into();
+        let expected = to_html("-P '127.0.0.1:420' 'http://127.0.0.1:420");
+        assert_eq!(to_be_tested, expected);
+    }
+}
